/******************************************************************************
   PMF.c

******************************************************************************/

// Include Files
#include <MC9S12E128.h>  // Derivative information
#include <stdio.h>
#include "PMF.h"
#include "ATD.h"
#include "Buffers.h"
#include "SCI0.h"


// Constants
// One-quarter of a sine wave (90 degrees) divided into 256 points
const byte Sine[256] =
{
  0,  2,  3,  5,  6,  8,  9, 11, 13, 14, 16, 17, 19, 20, 22, 24,
 25, 27, 28, 30, 31, 33, 35, 36, 38, 39, 41, 42, 44, 45, 47, 49,
 50, 52, 53, 55, 56, 58, 59, 61, 62, 64, 65, 67, 69, 70, 72, 73,
 75, 76, 78, 79, 81, 82, 84, 85, 87, 88, 90, 91, 92, 94, 95, 97,
 98,100,101,103,104,106,107,108,110,111,113,114,116,117,118,120,
121,122,124,125,127,128,129,131,132,133,135,136,137,139,140,141,
143,144,145,147,148,149,150,152,153,154,156,157,158,159,160,162,
163,164,165,167,168,169,170,171,172,174,175,176,177,178,179,180,
182,183,184,185,186,187,188,189,190,191,192,193,194,195,196,197,
198,199,200,201,202,203,204,205,206,207,208,209,210,211,212,213,
213,214,215,216,217,218,218,219,220,221,222,222,223,224,225,226,
226,227,228,228,229,230,231,231,232,233,233,234,235,235,236,236,
237,238,238,239,239,240,240,241,241,242,242,243,243,244,244,245,
245,246,246,247,247,247,248,248,249,249,249,250,250,250,251,251,
251,252,252,252,252,253,253,253,253,254,254,254,254,254,255,255,
255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
};


// Global Variables
byte Phase;  // Positive and negative half-cycle (2 per cycle)
byte Slope;  // Positive and negative slope (2 positive and 2 negative per cycle)
word Angle;  // Phase angle of a sign wave (1020 per revolution)
byte Amp;    // Amplitude of sine wave generated
int  Adj;    // Correction for zero crossing

word SineValue;    // Value of the sine wave from table



/*****************************************************************************/
#pragma CODE_SEG NON_BANKED
interrupt void PMFC_Reload_Int(void)
{
  if (Phase)  // Positive half-cycle
  {
    if (Slope)  // Positive slope
    {
      SineValue = Sine[Angle++];  // Get sine wave value from table
      PMFVAL4 = (Modulus/2)+(SineValue*Amp/256);  // Calculate PWM
      if (Angle == 255)
      {
        switch(Adj)
        {
          case +1:     // Add an extra angle cycle
            Angle--;   // Will do 254 over again
            Adj = 0;
            break;

          case 0:      // In sync, no need to adjust
            Slope = 0; // Change to negative slope
            break;

          case -1:     // Subtract one angle cycle
            Slope = 0; // Change to negative slope
            Angle--;   // Will not do 255
            Adj = 0;
            break;
        }
      }
    }
    else  // Negative slope
    {
      SineValue = Sine[Angle--];  // Get sine wave value from table
      PMFVAL4 = (Modulus/2)+(SineValue*Amp/256);  // Calculate PWM
      if (Angle == 0)
        Phase = 0;  // Change to negative half-cycle
    }
  }
  else  // Negative half-cycle
  {
    if (!Slope)  // Negative slope
    {
      SineValue = Sine[Angle++];  // Get sine wave value from table
      PMFVAL4 = (Modulus/2)-(SineValue*Amp/256);  // Calculate PWM
      if (Angle == 255)
      {
        switch(Adj)
        {
          case +1:     // Add an extra angle cycle
            Angle--;   // Will do 254 over again
            Adj = 0;
            break;

          case 0:      // In sync, no need to adjust
            Slope = 1; // Change to positive slope
            break;

          case -1:     // Subtract one angle cycle
            Slope = 1; // Change to positive slope
            Angle--;   // Will not do 255
            Adj = 0;
            break;
        }
      }
    }
    else  // Positive slope
    {
      SineValue = Sine[Angle--];  // Get sine wave value from table
      PMFVAL4 = (Modulus/2)-(SineValue*Amp/256);  // Calculate PWM
      if (Angle == 0)
        Phase = 1;  // Change to positive half-cycle
    }
  }
  PMFENCC_LDOKC = 1;   // Allow PWM register load
  PMFFQCC_PWMRFC = 1;  // Clear PWM Reload Flag C
}
#pragma CODE_SEG DEFAULT


/*****************************************************************************/
void InitPMF(void)
{
/* Use multiple timebase generators. C, B, and A are edge-aligned. C is
   complementary pair, B and A are independent. */
  PMFCFG0 = PMFCFG0_MTG_MASK | PMFCFG0_EDGEC_MASK | PMFCFG0_EDGEB_MASK |
            PMFCFG0_EDGEA_MASK |/* PMFCFG0_INDEPC_MASK |*/ PMFCFG0_INDEPB_MASK |
            PMFCFG0_INDEPA_MASK;

// Prescaler C
  PMFFQCC_PRSCC = 0;  // PWM clock frequency is f(bus)

// Load frequency
  PMFFQCC_LDFQC = 0;  // Reload every PWM opportunity

// Modulo of PWM C
  PMFMODC = Modulus;  // See PMF.h

// Pulse width of PWM C
  PMFVAL4 = Modulus/2;  // Equal positive and negative periods produce 0 volts

// Deadtime of PWM C
  PMFDTMC = 2;        // Insert this many f(bus) cycles for dead time

// Enable generator C
  PMFENCC_LDOKC = 1;
  PMFENCC_PWMENC = 1;

  Phase = 1;   // Phase angle of sine wave
  Slope = 1;   // 1 = going up, 0 = going down
  Angle = 0;   // Start at zero crossing
  Amp = 194;   // Maximum amplitude for test
  //Amp = 0;     // Starting amplitude is zero

  PMFENCC_PWMRIEC = 1;  // Enable PWM C interrupts
}


/*****************************************************************************/
void IncAmplitude(void)
{
  word Test1, Test2;

  Test1 = Amp+1;  // Test to see if amplitude can be increased
  Test2 = (Modulus/2)+(255*Test1/256);
  if (Test2 < Modulus-2)
    Amp++;
}


/*****************************************************************************/
void DecAmplitude(void)
{
  if (Amp > 0)
    Amp--;
}


/*****************************************************************************/
void DisplayAmplitude(void)
{
  char* BufNum;

  BufNum = GetBuffer();				  // Get a free buffer
  (void)sprintf(BufNum,"Amplitude: %d\r\n",Amp);
  SCI0_PutQueue(BufNum);
}

